// bc_amm.cpp : Defines the entry point for the DLL application.
//
// Auto Material Mapper:  Generates the material map for the level.
//
// BattleCraft 1942 and BattleField 1942 are registered trademarks of
// Digital Illusions CE.  (c)2003 by Digital Illusions CE.
// All Right Reserved.
//
// Programmed by Eric PL Smith (aka Zerk) of Red Monkey Studios
//
/* MATERIAL MAP DATA BY TYPE
0 Default
1 Water/Ice/Slippery
2 Dry Grass
3 Wet Grass
4 Dry Dirt
5 Wet Dirt
6 Mud
7 Outside Map
8 Gravel
9 Frozen Ground
10 Dry Sand
11 Wet Sand
12 Rock Surface
13 Sand Road
14 Dirt Road
15 Paved Road
*/

#include "stdafx.h"
#include "resource.h"
#include <malloc.h>
#include <stdlib.h>

#include "bc_amm.h"

#define MODE_BYHEIGHT			0
#define MODE_BYANGLE			1

LRESULT CALLBACK DialogWndProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);

HINSTANCE hInstance;

char module[MAXMODULE];

hmap_s hmap[1033][1033];

matmap_s mmap[1033][1033];

int mode = MODE_BYHEIGHT;

//The main entry point of the DLL.  This is called when the DLL is
//loaded and is called when BattleCraft 1942 first starts up.
BOOL APIENTRY DllMain( HINSTANCE hinst, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
	hInstance = hinst;
    return TRUE;
}

//initMatMap()
// This is called then the user selects the "Generate Material Map" option
// from within BC42 and is called by BC42.  Return TRUE to continue the
// generation process or return FALSE and BC42 will abort the process.
//
// matmap = a pointer to the material map data stored in a multidimensional array
// heightmap = a pointer to the terrain heightmap data
//
extern "C" __declspec(dllexport) bool initMatMap(void) {
	bool result = true;

	//user defined dialog.
	int ret = DialogBox(hInstance, (LPCTSTR)IDD_AMM_DIALOG, NULL, (DLGPROC)DialogWndProc);
	if(ret != IDOK)
		return false;

	GetModuleFileName(NULL, (LPTSTR)module, MAXMODULE);
	
	return result;
}

// generateMatMap()
// This is called to generate the material map
//
// size = The dimensions of the material map to generate.
//
int MatHeights[6] = {25, 70, 80, 127, 200, 255}; // terrain height in BF1942 is 0-255
int MatMaterial[6] = {11, 10, 2, 3, 12, 9}; // use a material from the material index

extern "C" __declspec(dllexport) bool generateMatMap(int size, matmap_s *matmap, hmap_s *htmap, float waterLevel, float heightScale)
{
	//copy the terrain heightmap data to a new memory location
	memcpy(hmap, htmap, (1033*1033)*sizeof(hmap_s));

	//copy the material map data to a new memory location
	memcpy(mmap, matmap, (1033*1033)*sizeof(matmap_s));

	//Generate the terrain
	switch (mode)
	{
	default:
	case MODE_BYHEIGHT:
		{
			float lowest = 2000;
			float highest = -2000;

			for (int x = 0; x < size; x++)
			{
				for (int z = 0; z < size; z++)
				{
					float height = hmap[x][z].y;
					if(height > highest) highest = height;
					if(height < lowest) lowest = height;

					if(mmap[x][z].type < 13) // dont redo already placed road.
					{
						for (int i = 0; i < 6; i++)
						{
							if (i > 0)
							{
								if (height <= MatHeights[i] && height > MatHeights[i-1])
								mmap[x][z].type = MatMaterial[i];
							}
							else
							{
								if (height <= MatHeights[i])
								mmap[x][z].type = MatMaterial[i];
							}
						}
						// lets put sand down under and around the water
						if(height*heightScale <= waterLevel+1)
							mmap[x][z].type = 11;
					}
				}
			}
			break;
		}
	case MODE_BYANGLE:
		{
			//see if the side is steep enough to add a rocky material
			//nothing spectacular going on here.
			for (int x = 1; x <= size; x++)
			{
				for (int z = 1; z <= size; z++)
				{
					float height = hmap[x][z].y*heightScale;
					float height2 = hmap[x-1][z+1].y*heightScale;

					int diff = abs((int)height - (int)height2);

					if(mmap[x][z].type < 13) // dont redo already placed road.
					{
						if(diff >= 2)
						{
							mmap[x][z].type = 12;
						}
						else
						{
							mmap[x][z].type = 3;
						}

						// put sand down under and around the water
						if(height <= waterLevel+1)
							mmap[x][z].type = 11;
					}
				}
			}
			break;
		}
	}

	//copy the new data back to the material map
	memcpy(matmap, mmap, (1033*1033)*sizeof(matmap_s));

	//return our material map back to BC42
	return true;
}

LRESULT CALLBACK DialogWndProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_INITDIALOG:
			{
				CheckDlgButton( hDlg, IDC_METHOD_BYHEIGHT, TRUE );
				mode = MODE_BYHEIGHT;
				return TRUE;
			}
		case WM_DESTROY:
			{
				EndDialog(hDlg, IDOK);
				return TRUE;
			}

		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
			case IDOK:
				{
					if (IsDlgButtonChecked(hDlg, IDC_METHOD_BYHEIGHT) == BST_CHECKED)
					{
						mode = MODE_BYHEIGHT;
					}
					else if (IsDlgButtonChecked(hDlg, IDC_METHOD_BYANGLE) == BST_CHECKED)
					{
						mode = MODE_BYANGLE;
					}
					EndDialog(hDlg, IDOK);
					return TRUE;
				}
			case IDCANCEL:
				{
					EndDialog(hDlg, IDCANCEL);
					return TRUE;
				}
			case IDC_METHOD_BYHEIGHT:
				CheckDlgButton( hDlg, IDC_METHOD_BYHEIGHT, TRUE );
				CheckDlgButton( hDlg, IDC_METHOD_BYANGLE, FALSE );
				return true;
			case IDC_METHOD_BYANGLE:
				CheckDlgButton( hDlg, IDC_METHOD_BYHEIGHT, FALSE );
				CheckDlgButton( hDlg, IDC_METHOD_BYANGLE, TRUE );
				return true;
			}
			break;
	}
    return FALSE;
}

